import json
from collections import namedtuple
from django.shortcuts import render
from django.contrib.auth import login, logout
from django.core.exceptions import ObjectDoesNotExist
from django.http import JsonResponse, HttpResponseRedirect, HttpResponse
from django.contrib.auth.models import User
from django.contrib.auth import authenticate,login, logout
from django.contrib.auth.decorators import login_required
from django.shortcuts import render
from django.views.decorators.csrf import csrf_exempt
from django.core.cache import cache
from mongo_datatables import DataTables
from .models import *
from pymongo import MongoClient
import datetime
from bson import ObjectId
from django.http.response import JsonResponse
from django.conf import settings
import uuid
import sys
from django.contrib.auth.hashers import make_password
from django.core.files.storage import default_storage
from django.contrib import messages




def LoginView(request):
    isLogin = request.session.get('is_authorized')
    if request.POST :
        redirect_to = request.GET.get('next', '')
        username = request.POST.get('uName')
        passw = request.POST.get('password')
        user = authenticate(username=username, password=passw)
        if user is not None:
            if user.is_active:
                login(request,user)
                request.session["username"]=user.username
                request.session["is_authorized"]=True
                request.session["is_superuser"]=user.is_superuser
                request.session["is_staff"]=user.is_staff
                messages.success(request, 'You have login successfully')
                if redirect_to:
                    return HttpResponseRedirect(redirect_to)
                else :
                    return HttpResponseRedirect('/adminnct/dashboard')
            else:
                messages.error(request, 'User is not activated Yet')
                return HttpResponseRedirect('/adminnct/login')
        else:
            messages.error(request, 'You have entered wrong credentials')
            return HttpResponseRedirect('/adminnct/login')
    if isLogin :
        return HttpResponseRedirect('/adminnct/dashboard')
    else :  
        return render(request, "login-nct-tpl.html")

@login_required(login_url='/adminnct/')
def DashboardView(request):
    return render(request, "dashboard-nct-tpl.html")

def ForgetPasswordView(request):
    return render(request, "forget-password-nct-tpl.html")

@login_required(login_url='/adminnct/')
def ChangePasswordView(request):
    return render(request, "change-password-nct-tpl.html")

@login_required(login_url='/adminnct/')
def ManagePatentsView(request):
    return render(request, "manage-patents-nct-tpl.html")

@csrf_exempt
def ManagePatentDataView(request):
    columns = ["UUID", 'patentTitle', 'applicatitionNumber', 'filingDate',"inventors"]
    index_column = "UUID"
    collection = "adminnct_patents"
    results = DataTablesServer(request, columns, index_column, collection).output_result()
    resultData = results['aaData']
    aaData_updated_rows = []
    for x in resultData:
        viewBtn = '<a href="javascript:void(0)" class="btn btn-info btn-icon-split btn_view_action" data-table="Patents" data-id="'+x[0]+'"><span class="icon text-white-50"><i class="fas fa-eye"></i></span><span class="text">View</span></a>'
        deleteBtn = '<a href="javascript:void(0)" class="btn btn-danger btn-icon-split btn_delete_action" data-table="Patents" data-id="'+x[0]+'"><span class="icon text-white-50"><i class="fas fa-trash"></i></span><span class="text">Delete</span></a>'
        operationBtn = viewBtn +' ' + deleteBtn
        date_time_obj = datetime.datetime.strptime(x[3], '%Y-%m-%d %H:%M:%S.%f')
        x[3] = date_time_obj.strftime(settings.DATETIME_FORMAT)
        x.append(operationBtn)
        aaData_updated_rows.append(x)
    results['aaData'] = aaData_updated_rows;
    return JsonResponse(results)

@login_required(login_url='/adminnct/')
def ManagePaymentView(request):
    return render(request, "manage-payments-nct-tpl.html")

@csrf_exempt
def ManagePaymentDataView(request):
    columns = ["UUID", 'userUUID','paymentAmount', 'transactionId','payment_on']
    index_column = "UUID"
    collection = "adminnct_paymenthistory"
    results = DataTablesServer(request, columns, index_column, collection).output_result()
    resultData = results['aaData']
    aaData_updated_rows = []
    for x in resultData:
        date_time_obj = datetime.datetime.strptime(x[4], '%Y-%m-%d %H:%M:%S.%f')
        x[4] = date_time_obj.strftime(settings.DATETIME_FORMAT)
        userDetails = Users.objects.get(UUID=x[1])
        x[1] = userDetails.firstName +" "+userDetails.lastName
        aaData_updated_rows.append(x)
    results['aaData'] = aaData_updated_rows;
    return JsonResponse(results)

@login_required(login_url='/adminnct/')
def ManageUsersView(request):
    # ContactData = Users()
    # ContactData.UUID = uuid.uuid1();
    # ContactData.firstName = 'Milan';
    # ContactData.lastName = 'Hirpara';
    # ContactData.emailAddress = 'pratik.ranpara@ncrypted.com';
    # ContactData.password = make_password('123456');
    # ContactData.isActive = True;
    # ContactData.isverified = False;
    # ContactData.created_on = datetime.datetime.now();
    # ContactData.updated_on = datetime.datetime.now();
    # ContactData.save()
    return render(request, "manage-users-nct-tpl.html")

@csrf_exempt
def ManageUserDataView(request):
    columns = ["UUID", 'profileImage','firstName', 'lastName', 'emailAddress','created_on',"isActive"]
    index_column = "UUID"
    collection = "adminnct_users"
    results = DataTablesServer(request, columns, index_column, collection).output_result()
    resultData = results['aaData']
    aaData_updated_rows = []
    for x in resultData:
        editBtn = '<a href="manage-user-form/'+x[0]+'" class="btn btn-success btn-icon-split"><span class="icon text-white-50"><i class="fas fa-edit"></i></span><span class="text">Edit</span></a>'
        deleteBtn = '<a href="javascript:void(0)" class="btn btn-danger btn-icon-split btn_delete_action" data-table="Users" data-id="'+x[0]+'"><span class="icon text-white-50"><i class="fas fa-trash"></i></span><span class="text">Delete</span></a>'
        operationBtn = editBtn +' ' + deleteBtn
        if x[6] == 'True' :
            status = "checked" 
        else :
            status = ""
        statusSwitch = "<div class='switch-small'><input type='checkbox' name='' id='' class='make-switch' data-table='Users' data-id='"+x[0]+"' "+status+" data-on-label='<i class="'"fa fa-check"'"></i>' data-off-label='<i class="'"fa fa-times"'" ></i>'></div>"
        date_time_obj = datetime.datetime.strptime(x[5], '%Y-%m-%d %H:%M:%S.%f')
        x[5] = date_time_obj.strftime(settings.DATETIME_FORMAT)
        x[6] = statusSwitch
        if x[1] :
            x[1] = "<img src="+settings.MEDIA_URL+"/"+x[1]+" height='50px' width='50px'>"
        else :
            x[1] = "<img src="+settings.MEDIA_URL+"/no_image_thumb.png height='50px' width='50px'>"
        x.append(operationBtn)
        aaData_updated_rows.append(x)
    results['aaData'] = aaData_updated_rows;
    return JsonResponse(results)

@login_required(login_url='/adminnct/')
def ManageUserFormView(request,user_id):
    if request.POST :
        postData = request.POST
        if user_id:
            userDetails = Users.objects.filter(UUID=user_id)
            if userDetails:
                updateUserDetail = Users.objects.get(UUID=user_id)
                if postData['isActive'] == 'n' :
                    isActive = False
                else :
                    isActive = True

                updateUserDetail.firstName =  postData['firstName']
                updateUserDetail.lastName =  postData['lastName']
                updateUserDetail.isActive =  isActive
                
                filepath = request.FILES.get('profileImage', False)
                if filepath:
                    updateUserDetail.profileImage = request.FILES['profileImage']
                updateUserDetail.updated_on = datetime.datetime.now(); 
                updateUserDetail.save()
                messages.success(request, 'User details updated successfully')
            else :
                messages.error(request, 'User details does not exists')

            return HttpResponseRedirect("/adminnct/manage-users")    
    else :
        isActive_a = ''
        isActive_d = ''
        proImg = 'required'
        if user_id :
            userDetails = Users.objects.filter(UUID=user_id)
            if userDetails:
                userDetails = Users.objects.get(UUID=user_id)
                firstName = userDetails.firstName
                lastName = userDetails.lastName
                emailAddress = userDetails.emailAddress
                profileImage = userDetails.profileImage
                if profileImage :
                    proImg = ''   
                if userDetails.isActive :
                    isActive_a = 'checked'
                else :
                    isActive_d = 'checked'
                return render(request, 'manage-user-form-nct-tpl.html',{'proImg':proImg,'firstName':firstName,'lastName':lastName,'emailAddress':emailAddress,'profileImage':profileImage,'isActive_a':isActive_a,'isActive_d':isActive_d,'media_url':settings.MEDIA_URL})       
            else :
                messages.error(request, 'User details does not exists')
                return HttpResponseRedirect("/adminnct/manage-users")
        else :
            messages.error(request, 'User details does not exists')
            return HttpResponseRedirect("/adminnct/manage-users")   
    
@login_required(login_url='/adminnct/')
def ContactUsView(request):
    # ContactData = ContactUs()
    # ContactData.UUID = uuid.uuid1();
    # ContactData.firstName = 'Pratik';
    # ContactData.lastName = 'Ranpara';
    # ContactData.emailAddress = 'milan.hirpara@ncrypted.com';
    # ContactData.subject = 'Just for testing';
    # ContactData.message = 'Just for testing the details';
    # ContactData.replayMessage = '';
    # ContactData.created_on = datetime.datetime.now();
    # ContactData.updated_on = datetime.datetime.now();
    # ContactData.save()
    return render(request, "contact-us-nct-tpl.html")

@login_required(login_url='/adminnct/')
def SiteSettingView(request):
    if request.POST :
        postData = request.POST
        siteSettingData = SiteSettings.objects.get(UUID=settings.SITE_SETTING_ID)
        siteSettingData.siteName =postData['siteName']
        logoPath = request.FILES.get('siteLogo', False)
        if logoPath:
            siteSettingData.siteLogo = request.FILES['siteLogo']

        faviconPath = request.FILES.get('siteFavicon', False)
        if faviconPath:
            siteSettingData.siteFavicon = request.FILES['siteFavicon']
        siteSettingData.adminName =postData['adminName']
        siteSettingData.adminEmail =postData['adminEmail']
        siteSettingData.googleApiKey =postData['googleApiKey']
        siteSettingData.googleSecretKey =postData['googleSecretKey']
        siteSettingData.fromEmail =postData['fromEmail']
        siteSettingData.smtpHost =postData['smtpHost']
        siteSettingData.smtpPort =postData['smtpPort']
        siteSettingData.smtpEmail =postData['smtpEmail']
        siteSettingData.smtpPassword =postData['smtpPassword']
        siteSettingData.facebookLink =postData['facebookLink']
        siteSettingData.twitterLink =postData['twitterLink']
        siteSettingData.googleLink =postData['googleLink']
        siteSettingData.linkedinLink =postData['linkedinLink']
        siteSettingData.paypalPaymentId =postData['paypalPaymentId']
        siteSettingData.updated_on = datetime.datetime.now();
        siteSettingData.save()
        messages.success(request, 'Site setting details has been updated successfully')
        return HttpResponseRedirect("/adminnct/site-settings")  
    logoExists = 'required';
    faviconExists = 'required';
    siteSettingDetails = SiteSettings.objects.filter(UUID=settings.SITE_SETTING_ID)
    if siteSettingDetails : 
        siteSettingDetails = SiteSettings.objects.get(UUID=settings.SITE_SETTING_ID)
        # print ("siteSettingDetails.siteFavicon",siteSettingDetails.siteFavicon)
        # logoImage = default_storage.exists(settings.MEDIA_ROOT +"/"+str(siteSettingDetails.siteLogo))
        # faviconImage = default_storage.exists(settings.MEDIA_ROOT +"/"+str(siteSettingDetails.siteFavicon))
        if siteSettingDetails.siteLogo : 
           logoExists = '';
        if siteSettingDetails.siteFavicon : 
           faviconExists = ''; 
    else :
        siteSettingDetails = ''
    return render(request, "site-settings-nct-tpl.html",{'logoExists':logoExists,'faviconExists':faviconExists,'settings':siteSettingDetails,'media_url':settings.MEDIA_URL})

@csrf_exempt
def ContactUsDataView(request):
    columns = ["UUID", 'firstName', 'lastName', 'emailAddress',"subject","message","replayMessage","created_on"]
    index_column = "UUID"
    collection = "adminnct_contactus"
    results = DataTablesServer(request, columns, index_column, collection).output_result()
    resultData = results['aaData']
    aaData_updated_rows = []
    for x in resultData:
        replyBtn = '<a href="reply-contact-form/'+x[0]+'" class="btn btn-success btn-icon-split"><span class="icon text-white-50"><i class="fas fa-edit"></i></span><span class="text">Reply</span></a>'
        deleteBtn = '<a href="javascript:void(0)" class="btn btn-danger btn-icon-split btn_delete_action" data-table="ContactUs" data-id="'+x[0]+'"><span class="icon text-white-50"><i class="fas fa-trash"></i></span><span class="text">Delete</span></a>'
        operationBtn = replyBtn + ' ' + deleteBtn
        date_time_obj = datetime.datetime.strptime(x[7], '%Y-%m-%d %H:%M:%S.%f')
        x[7] = date_time_obj.strftime(settings.DATETIME_FORMAT)
        x.append(operationBtn)
        aaData_updated_rows.append(x)
    results['aaData'] = aaData_updated_rows;
    return JsonResponse(results)

@login_required(login_url='/adminnct/')
def ReplyContactFormView(request,contact_id):
    if request.POST :
        postData = request.POST
        if postData['contactId']:
            updateReplyDetail = ContactUs.objects.get(UUID=contact_id)
            updateReplyDetail.replayMessage =  postData['replayMessage']
            updateReplyDetail.updated_on = datetime.datetime.now(); 
            updateReplyDetail.save()
            messages.success(request, 'Replay has been sended successfully.')
            return HttpResponseRedirect("/adminnct/contact-us") 
    contactFromDetails = ContactUs.objects.get(UUID=contact_id)
    return render(request, 'contact-reply-form-nct-tpl.html',{'formDetails':contactFromDetails})

@login_required(login_url='/adminnct/')
def ManageContentView(request):
    return render(request, "manage-content-nct-tpl.html")

@csrf_exempt
def ManageContentDataView(request):
    columns = ["UUID", 'pageName', 'pageTitle', 'created_on',"isActive"]
    index_column = "UUID"
    collection = "adminnct_content"
    results = DataTablesServer(request, columns, index_column, collection).output_result()
    resultData = results['aaData']
    aaData_updated_rows = []
    for x in resultData:
        editBtn = '<a href="manage-content-form/'+x[0]+'" class="btn btn-success btn-icon-split"><span class="icon text-white-50"><i class="fas fa-edit"></i></span><span class="text">Edit</span></a>'
        viewBtn = '<a href="javascript:void(0)" class="btn btn-info btn-icon-split btn_view_action" data-table="Content" data-id="'+x[0]+'"><span class="icon text-white-50"><i class="fas fa-eye"></i></span><span class="text">View</span></a>'
        deleteBtn = '<a href="javascript:void(0)" class="btn btn-danger btn-icon-split btn_delete_action" data-table="Content" data-id="'+x[0]+'"><span class="icon text-white-50"><i class="fas fa-trash"></i></span><span class="text">Delete</span></a>'
        operationBtn = editBtn +' '+ viewBtn +' ' + deleteBtn
        if x[4] == 'True' :
            status = "checked" 
        else :
            status = ""
        print ("status",status)
        statusSwitch = "<div class='switch-small'><input type='checkbox' name='' id='' class='make-switch' data-table='Content' data-id='"+x[0]+"' "+status+" data-on-label='<i class="'"fa fa-check"'"></i>' data-off-label='<i class="'"fa fa-times"'" ></i>'></div>"
        date_time_obj = datetime.datetime.strptime(x[3], '%Y-%m-%d %H:%M:%S.%f')
        x[3] = date_time_obj.strftime(settings.DATETIME_FORMAT)
        x[4] = statusSwitch
        x.append(operationBtn)
        aaData_updated_rows.append(x)
    results['aaData'] = aaData_updated_rows;
    return JsonResponse(results)

@login_required(login_url='/adminnct/')
def ManageContentFormView(request,content_id=''):
    
    if request.POST :
        postData = request.POST
        if postData['actionType'] == 'Add' :
            storeContentData = Content()
            storeContentData.UUID = uuid.uuid1()
            storeContentData.created_on = datetime.datetime.now()
        else :
            storeContentData = Content.objects.get(UUID=content_id)
        if postData['isActive'] == 'n' :
            isActive = False
        else :
            isActive = True
        
        storeContentData.pageName = postData['pageName']
        storeContentData.pageTitle = postData['pageTitle']
        storeContentData.pageSlug = postData['pageName']
        storeContentData.pageDescription = postData['pageDescription']
        storeContentData.metaKeyword = postData['metaKeyword']
        storeContentData.metaDescription =postData['metaDescription']
        storeContentData.isActive = isActive
        storeContentData.updated_on = datetime.datetime.now()
        storeContentData.save()
        if postData['actionType'] == 'Add' :
            messages.success(request, 'Content data added successfully')
        else :
            messages.success(request, 'Content data updated successfully')
        return HttpResponseRedirect("/adminnct/manage-content")
    else :
        isActive_a = ''
        isActive_d = ''
        if content_id :
            contentFormDetails = Content.objects.filter(UUID=content_id)
            if contentFormDetails:
                contentFormDetails = Content.objects.get(UUID=content_id)
                actionType = 'Edit'
                UUID = contentFormDetails.UUID
                pageName = contentFormDetails.pageName
                pageTitle = contentFormDetails.pageTitle
                pageDescription = contentFormDetails.pageDescription
                metaKeyword = contentFormDetails.metaKeyword
                metaDescription = contentFormDetails.metaDescription
                if contentFormDetails.isActive :
                    isActive_a = 'checked'
                else :
                    isActive_d = 'checked'       
            else :
                messages.error(request, 'Content data does not exists')
                return HttpResponseRedirect("/adminnct/manage-content")    
            
        else:
            actionType = 'Add'
            UUID = ""
            pageName = ""
            pageTitle = ""
            pageDescription = ""
            metaKeyword = ""
            metaDescription = ""
            isActive_a = 'checked'
            isActive_d = ''
        return render(request, 
            'manage-content-form-nct-tpl.html',
            {'actionType':actionType,'UUID':UUID,'pageName':pageName,
            'pageTitle':pageTitle,'pageDescription':pageDescription,'metaKeyword':metaKeyword,
            'metaDescription':metaDescription,'isActive_a':isActive_a,'isActive_d':isActive_d
            })

@login_required(login_url='/adminnct/')
def ManageTemplatesView(request):
    # ContactData = EmailTemplates()
    # ContactData.UUID = uuid.uuid1();
    # ContactData.constant = 'contact_query';
    # ContactData.types = 'Contact Us Query';
    # ContactData.subject = 'milan.hirpara@ncrypted.com';
    # ContactData.description = 'user registration';
    # ContactData.templates = 'Contact Us replay from admin';
    # ContactData.isActive = True;
    # ContactData.updated_on = datetime.datetime.now();
    # ContactData.save()
    return render(request, "manage-templates-nct-tpl.html")

@csrf_exempt
def ManageTemplatesDataView(request):
    columns = ["UUID", 'constant', 'types', 'description',"isActive"]
    index_column = "UUID"
    collection = "adminnct_emailtemplates"
    results = DataTablesServer(request, columns, index_column, collection).output_result()
    resultData = results['aaData']
    aaData_updated_rows = []
    for x in resultData:
        editBtn = '<a href="manage-templates-form/'+x[0]+'" class="btn btn-success btn-icon-split"><span class="icon text-white-50"><i class="fas fa-edit"></i></span><span class="text">Edit</span></a>'
        operationBtn = editBtn
        if x[4] == 'True' :
            status = "checked" 
        else :
            status = ""
        print ("status",status)
        statusSwitch = "<div class='switch-small'><input type='checkbox' name='' id='' class='make-switch' data-table='EmailTemplates' data-id='"+x[0]+"' "+status+" data-on-label='<i class="'"fa fa-check"'"></i>' data-off-label='<i class="'"fa fa-times"'" ></i>'></div>"
        x[4] = statusSwitch
        x.append(operationBtn)
        aaData_updated_rows.append(x)
    results['aaData'] = aaData_updated_rows;
    return JsonResponse(results)

@login_required(login_url='/adminnct/')
def ManageTemplatesFormView(request,template_id=''):
    
    if request.POST :
        postData = request.POST

        updateEmailTemplate = EmailTemplates.objects.get(UUID=template_id)
        updateEmailTemplate.subject = postData['subject']
        updateEmailTemplate.description = postData['description']
        updateEmailTemplate.templates = postData['templates']
        updateEmailTemplate.updated_on = datetime.datetime.now()
        updateEmailTemplate.save()
        
        messages.success(request, 'Email template data updated successfully')
        return HttpResponseRedirect("/adminnct/manage-templates")

    templateFormDetails = EmailTemplates.objects.filter(UUID=template_id)
    if templateFormDetails:
        templateFormDetails = EmailTemplates.objects.get(UUID=template_id)
        actionType = 'Edit'
        UUID = templateFormDetails.UUID
        subject = templateFormDetails.subject
        description = templateFormDetails.description
        templates = templateFormDetails.templates       
    else :
        messages.error(request, 'Email template does not exists')
        return HttpResponseRedirect("/adminnct/manage-templates")    
            
    return render(request, 'manage-templates-form-nct-tpl.html',
        {'actionType':actionType,'UUID':UUID,'subject':subject,
        'description':description,'templates':templates})

@login_required(login_url='/adminnct/')
def ManageSubscriptionPlansView(request):
    return render(request, "manage-subscription-plans-nct-tpl.html")

@csrf_exempt
def ManageSubscriptionPlansDataView(request):
    columns = ["UUID", 'planName', 'planPrice', 'created_on',"isActive"]
    index_column = "UUID"
    collection = "adminnct_subscriptionplans"
    results = DataTablesServer(request, columns, index_column, collection).output_result()
    resultData = results['aaData']
    aaData_updated_rows = []
    for x in resultData:
        editBtn = '<a href="manage-subscription-plans-form/'+x[0]+'" class="btn btn-success btn-icon-split"><span class="icon text-white-50"><i class="fas fa-edit"></i></span><span class="text">Edit</span></a>'
        viewBtn = '<a href="javascript:void(0)" class="btn btn-info btn-icon-split btn_view_action" data-table="SubscriptionPlans" data-id="'+x[0]+'"><span class="icon text-white-50"><i class="fas fa-eye"></i></span><span class="text">View</span></a>'
        deleteBtn = '<a href="javascript:void(0)" class="btn btn-danger btn-icon-split btn_delete_action" data-table="SubscriptionPlans" data-id="'+x[0]+'"><span class="icon text-white-50"><i class="fas fa-trash"></i></span><span class="text">Delete</span></a>'
        operationBtn = editBtn +' '+ viewBtn +' ' + deleteBtn
        if x[4] == 'True' :
            status = "checked" 
        else :
            status = ""
        print ("status",status)
        statusSwitch = "<div class='switch-small'><input type='checkbox' name='' id='' class='make-switch' data-table='SubscriptionPlans' data-id='"+x[0]+"' "+status+" data-on-label='<i class="'"fa fa-check"'"></i>' data-off-label='<i class="'"fa fa-times"'" ></i>'></div>"
        date_time_obj = datetime.datetime.strptime(x[3], '%Y-%m-%d %H:%M:%S.%f')
        x[2] = settings.CURRENCY_SYMBOL + x[2]
        x[3] = date_time_obj.strftime(settings.DATETIME_FORMAT)
        x[4] = statusSwitch
        x.append(operationBtn)
        aaData_updated_rows.append(x)
    results['aaData'] = aaData_updated_rows;
    return JsonResponse(results)

@login_required(login_url='/adminnct/')
def ManageSubscriptionPlansFormView(request,plan_id=''):
    
    if request.POST :
        postData = request.POST
        if postData['actionType'] == 'Add' :
            storeSubscriptionPlansData = SubscriptionPlans()
            storeSubscriptionPlansData.UUID = uuid.uuid1()
            storeSubscriptionPlansData.created_on = datetime.datetime.now()
        else :
            storeSubscriptionPlansData = SubscriptionPlans.objects.get(UUID=plan_id)
        
        if postData['isActive'] == 'n' :
            isActive = False
        else :
            isActive = True

        if postData['selectLogin'] == 'n' :
            selectLogin = False
        else :
            selectLogin = True

        if postData['patentSearch'] == 'n' :
            patentSearch = False
        else :
            patentSearch = True
        
        storeSubscriptionPlansData.planName = postData['planName']
        storeSubscriptionPlansData.planPrice = postData['planPrice']
        storeSubscriptionPlansData.noOfPatentDetailView = postData['noOfPatentDetailView']
        storeSubscriptionPlansData.selectLogin = selectLogin
        storeSubscriptionPlansData.patentSearch = patentSearch
        storeSubscriptionPlansData.isActive = isActive
        storeSubscriptionPlansData.updated_on = datetime.datetime.now()
        storeSubscriptionPlansData.save()
        if postData['actionType'] == 'Add' :
            messages.success(request, 'Subscription plan added successfully')
        else :
            messages.success(request, 'Subscription plan updated successfully.')
        return HttpResponseRedirect("/adminnct/manage-subscription-plans")
    else :
        isActive_a = ''
        isActive_d = ''
        selectLogin_a = ''
        selectLogin_d = ''
        patentSearch_a = ''
        patentSearch_d = ''
        if plan_id :
            subscriptionPlanFormDetails = SubscriptionPlans.objects.filter(UUID=plan_id)
            if subscriptionPlanFormDetails:
                subscriptionPlanFormDetails = SubscriptionPlans.objects.get(UUID=plan_id)
                actionType = 'Edit'
                UUID = subscriptionPlanFormDetails.UUID
                planName = subscriptionPlanFormDetails.planName
                planPrice = subscriptionPlanFormDetails.planPrice
                noOfPatentDetailView = subscriptionPlanFormDetails.noOfPatentDetailView
                
                if subscriptionPlanFormDetails.isActive :
                    isActive_a = 'checked'
                else :
                    isActive_d = 'checked'  

                if subscriptionPlanFormDetails.selectLogin :
                    selectLogin_a = 'checked'
                else :
                    selectLogin_d = 'checked'  

                if subscriptionPlanFormDetails.patentSearch :
                    patentSearch_a = 'checked'
                else :
                    patentSearch_d = 'checked'       
            else :
                messages.error(request, 'Subscription plan does not exists')
                return HttpResponseRedirect("/adminnct/manage-subscription-plans")    
            
        else:
            actionType = 'Add'
            UUID = ""
            planName = ""
            planPrice = ""
            noOfPatentDetailView = ""
            isActive_a = 'checked'
            isActive_d = ''
            selectLogin_a = 'checked'
            selectLogin_d = ''
            patentSearch_a = 'checked'
            patentSearch_d = ''
        return render(request, 
            'manage-subscription-plan-form-nct-tpl.html',
            {'actionType':actionType,'UUID':UUID,'planName':planName,
            'planPrice':planPrice,'noOfPatentDetailView':noOfPatentDetailView,'selectLogin_a':selectLogin_a,
            'selectLogin_d':selectLogin_d,'patentSearch_a':patentSearch_a,'patentSearch_d':patentSearch_d,'isActive_a':isActive_a,'isActive_d':isActive_d
            })

@csrf_exempt
def DeleteRecords(request):
    recordId = request.POST['recordId']
    tableName = str_to_class(request.POST['tableName'])
    RemovedData = tableName.objects.get(UUID=recordId)
    deletedStatus = RemovedData.delete()
    if deletedStatus :
        result = {"status": True,"type":"success", "message": "Your selected record deleted successfully"}
    else:
        result = {"status": True,"type":"error", "message": "There seems to be an issue. Please try again after some time."}
    return JsonResponse(result)

@csrf_exempt
def ViewRecordDetails(request):
    recordId = request.POST['recordId']
    tableName = str_to_class(request.POST['tableName'])
    tableData = tableName.objects.get(UUID=recordId)
    contentData = ''
    if request.POST['tableName'] == 'Content':
        PageName = viewDetailHtmlContent("Page Name",tableData.pageName) 
        pageTitle =  viewDetailHtmlContent("Page Title",tableData.pageTitle)
        metaKeyword = viewDetailHtmlContent("Meta Keyword",tableData.metaKeyword)
        metaDescription = viewDetailHtmlContent("Meta Description",tableData.metaDescription)
        pageDescription = viewDetailHtmlContent("Page Description",tableData.pageDescription)
        contentData = PageName + pageTitle + metaKeyword + metaDescription + pageDescription
    if request.POST['tableName'] == 'SubscriptionPlans':
        planName = viewDetailHtmlContent("Plan Name",tableData.planName)
        planPrice = viewDetailHtmlContent("Plan Name",settings.CURRENCY_SYMBOL + str(tableData.planPrice))
        noOfPatentDetailView = viewDetailHtmlContent("No. of Patent Detail View",str(tableData.noOfPatentDetailView))
        
        if tableData.selectLogin :
            canLogin = 'Yes'
        else : 
            canLogin = 'No'

        if tableData.patentSearch :
            patentSearch = 'Yes'
        else : 
            patentSearch = 'No'

        if tableData.isActive :
            isActive = 'Active'
        else : 
            isActive = 'Inactive'
        selectLogin = viewDetailHtmlContent("Select Login",canLogin)
        patentSearch = viewDetailHtmlContent("Patent Search",patentSearch)
        isActive = viewDetailHtmlContent("Status",isActive)

        contentData = planName + planPrice + noOfPatentDetailView + selectLogin + patentSearch + isActive
    result = {"status": True,"contentData":contentData}    
    return JsonResponse(result)

def viewDetailHtmlContent(Title,Value):
    htmlData = '<div class="form-group col-md-12" id="123"><label class="control-label col-md-4">'+Title+'&nbsp;:</label><div class="col-md-7">'+Value+'</div></div>'
    return htmlData

@csrf_exempt
def ChangeRecordStatus(request):
    recordId = request.POST['recordId']
    statusValue = request.POST['statusValue']
    tableName = str_to_class(request.POST['tableName'])
    
    if statusValue == 'true':
        status = True
    else :
        status = False
    tableStatusData = tableName.objects.get(UUID=recordId)
    tableStatusData.isActive = status
    tableStatusData.updated_on = datetime.datetime.now(); 
    tableStatusData.save()
    # if tableStatusData.save() :
    if status:
        result = {"status": True,"type":"success", "message": "Record activated successfully"}
    else :
        result = {"status": True,"type":"success", "message": "Record deactivated successfully"}    
    # else:
    #     result = {"status": True,"type":"error", "message": "There seems to be an issue. Please try again after some time."}
    return JsonResponse(result)


def LogoutView(request):
    logout(request)
    messages.success(request, 'Logout successfully')
    return HttpResponseRedirect("/adminnct/login")
    # return render(request, "login-nct-tpl.html")

def str_to_class(str):
    return getattr(sys.modules[__name__], str)

order_dict = {'asc': 1, 'desc': -1}
class DataTablesServer(object):
 
    def __init__( self, request, columns, index, collection):
 
        self.columns = columns
        self.index = index
        self.collection = collection
         
        # values specified by the datatable for filtering, sorting, paging
        self.request_values = request.POST
         
        # connection to your mongodb (see pymongo docs). this is defaulted to localhost
        self.dbh = MongoClient(settings.PYMONGOHOSTLINK)
 
        # results from the db
        self.result_data = None
         
        # total in the table after filtering
        self.cardinality_filtered = 0
 
        # total in the table unfiltered
        self.cardinality = 0
 
        self.run_queries()

    def output_result(self):
                
        output = {}
        output['sEcho'] = str(int(self.request_values['sEcho']))
        output['iTotalRecords'] = str(self.cardinality)
        output['iTotalDisplayRecords'] = str(self.cardinality_filtered)
        aaData_rows = []
 
        for row in self.result_data:
            aaData_row = []
            for i in range( len(self.columns) ):
                # print (">>>>",type(row[ self.columns[i]]))
                aaData_row.append(str(row[ self.columns[i] ]).replace('"','\\"'))
             
            # add additional rows here that are not represented in the database
            # aaData_row.append(('''<input id='%s' type='checkbox'></input>''' % (str(row[ self.index ]))).replace('\\', ''))
 
            aaData_rows.append(aaData_row)
 
        output['aaData'] = aaData_rows
 
        return output

    def run_queries(self):
 
        # 'mydb' is the actual name of your database
        mydb = self.dbh.gracular
 
        # pages has 'start' and 'length' attributes
        pages = self.paging()
         
        # the term you entered into the datatable search
        _filter = self.filtering()
         
        # the document field you chose to sort
        sorting = self.sorting()
 
        # get result from db to display on the current page
        self.result_data = list(mydb[self.collection].find(filter = _filter,
                                                           skip = pages.start,
                                                           limit = pages.length,
                                                           sort = sorting))
        # print(">>>>>>>>>>>",self.result_data)
        # length of filtered set  
        self.cardinality_filtered = len(list(mydb[self.collection].find(filter = _filter)))
        
        # length of all results you wish to display in the datatable, unfiltered
        self.cardinality = len(list( mydb[self.collection].find()))
 
    def filtering(self):
         
        # build your filter spec
        _filter = {}
        if ( 'sSearch' in self.request_values ) and ( self.request_values['sSearch'] != "" ):
             
            # the term put into search is logically concatenated with 'or' between all columns
            or_filter_on_all_columns = []
             
            for i in range( len(self.columns) ):
                column_filter = {}
                # case insensitive partial string matching pulled from user input
                column_filter[self.columns[i]] = {'$regex': self.request_values['sSearch'], '$options': 'i'}
                or_filter_on_all_columns.append(column_filter)
            
            _filter['$or'] = or_filter_on_all_columns
        
        # individual column filtering - uncomment if needed
        
        #and_filter_individual_columns = []
        #for i in range(len(columns)):
        #    if (request_values.has_key('sSearch_%d' % i) and request_values['sSearch_%d' % i] != ''):
        #        individual_column_filter = {}
        #        individual_column_filter[columns[i]] = {'$regex': request_values['sSearch_%d' % i], '$options': 'i'}
        #        and_filter_individual_columns.append(individual_column_filter)

        #if and_filter_individual_columns:
        #    _filter['$and'] = and_filter_individual_columns
       
        return _filter
  
    def sorting(self):
                
        order = []
        # mongo translation for sorting order
        # if ( self.request_values['iSortCol_0'] != "" ) and ( self.request_values['iSortingCols'] > 0 ):
            
        for i in range( int(self.request_values['iSortingCols']) ):
            # column number
            column_number = int(self.request_values['iSortCol_'+str(i)])
            # sort direction
            sort_direction = self.request_values['sSortDir_'+str(i)]
            
            order.append((self.columns[column_number], order_dict[sort_direction]))
        
        return order
 
    def paging(self):
                
        pages = namedtuple('pages', ['start', 'length'])
        
        if (self.request_values['iDisplayStart'] != "" ) and (self.request_values['iDisplayLength'] != -1 ):
            pages.start = int(self.request_values['iDisplayStart'])
            pages.length = int(self.request_values['iDisplayLength'])
        
        return pages